@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,12 +21,19 @@ struct diff_state {
 	bool after_diff;
 	bool reading_diff_stat;
 	bool combined_diff;
+	bool adding_describe_ref;
+	const char *file;
+	unsigned int lineno;
+	struct position pos;
 };
 
 enum request diff_common_edit(struct view *view, enum request request, struct line *line);
 bool diff_common_read(struct view *view, const char *data, struct diff_state *state);
 enum request diff_common_enter(struct view *view, enum request request, struct line *line);
 struct line *diff_common_add_diff_stat(struct view *view, const char *text, size_t offset);
+void diff_common_select(struct view *view, struct line *line, const char *changes_msg);
+void diff_save_line(struct view *view, struct diff_state *state, enum open_flags flags);
+void diff_restore_line(struct view *view, struct diff_state *state);
 
 unsigned int diff_get_lineno(struct view *view, struct line *line);
 const char *diff_get_pathname(struct view *view, struct line *line);
