@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,18 +17,10 @@
 #include "tig/tig.h"
 #include "tig/keys.h"
 
-enum input_status {
-	INPUT_OK,
-	INPUT_SKIP,
-	INPUT_DELETE,
-	INPUT_STOP,
-	INPUT_CANCEL
-};
-
-int get_input(int prompt_position, struct key *key, bool modifiers);
+int get_input(int prompt_position, struct key *key);
+int get_input_char(void);
 
 extern WINDOW *status_win;
-extern FILE *opt_tty;
 
 void update_status(const char *msg, ...);
 void report(const char *msg, ...) PRINTF_LIKE(1, 2);
@@ -45,7 +37,7 @@ extern unsigned int current_view;
 #define foreach_displayed_view(view, i) \
 	for (i = 0; i < ARRAY_SIZE(display) && (view = display[i]); i++)
 
-#define displayed_views()	(display[1] != NULL ? 2 : 1)
+#define displayed_views()	(!!display[0] + !!display[1])
 
 #define view_is_displayed(view) \
 	(view == display[0] || view == display[1])
@@ -53,10 +45,19 @@ extern unsigned int current_view;
 void init_display(void);
 void resize_display(void);
 void redraw_display(bool clear);
+bool save_display(const char *path);
 
-bool open_external_viewer(const char *argv[], const char *dir, bool confirm, bool refresh, const char *notice);
+bool vertical_split_is_enabled(enum vertical_split vsplit, int height, int width);
+int apply_vertical_split(int base_width);
+
+bool open_external_viewer(const char *argv[], const char *dir, bool silent, bool confirm, bool refresh, const char *notice);
 void open_editor(const char *file, unsigned int lineno);
 void enable_mouse(bool enable);
 
+enum status_code open_script(const char *path);
+
+#define get_cursor_pos(cursor_y, cursor_x) getyx(newscr, cursor_y, cursor_x)
+#define set_cursor_pos(cursor_y, cursor_x) wmove(newscr, cursor_y, cursor_x)
+
 #endif
 /* vim: set ts=8 sw=8 noexpandtab: */
