@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,9 @@ struct ref;
 
 #define LINE_INFO(_) \
 	_(DIFF_HEADER,  	"diff --"), \
+	_(DIFF_DEL_FILE,  	"--- "), \
+	_(DIFF_ADD_FILE,  	"+++ "), \
+	_(DIFF_START,   	"---"), \
 	_(DIFF_CHUNK,   	"@@"), \
 	_(DIFF_ADD,		"+"), \
 	_(DIFF_ADD2,		" +"), \
@@ -79,7 +82,15 @@ struct ref;
 	_(PALETTE_4,		""), \
 	_(PALETTE_5,		""), \
 	_(PALETTE_6,		""), \
-	_(GRAPH_COMMIT,		"")
+	_(PALETTE_7,		""), \
+	_(PALETTE_8,		""), \
+	_(PALETTE_9,		""), \
+	_(PALETTE_10,		""), \
+	_(PALETTE_11,		""), \
+	_(PALETTE_12,		""), \
+	_(PALETTE_13,		""), \
+	_(GRAPH_COMMIT,		""), \
+	_(SEARCH_RESULT,	"")
 
 enum line_type {
 #define DEFINE_LINE_ENUM(type, line) LINE_##type
@@ -109,6 +120,9 @@ struct line_info *get_line_info(const char *prefix, enum line_type type);
 struct line_info *add_line_rule(const char *prefix, struct line_rule *rule);
 void init_colors(void);
 
+typedef bool (*line_rule_visitor_fn)(void *data, const struct line_rule *rule);
+bool foreach_line_rule(line_rule_visitor_fn fn, void *data);
+
 /* Color IDs must be 1 or higher. [GH #15] */
 #define COLOR_ID(line_type)		((line_type) + 1)
 
