@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,14 @@
 struct view;
 struct input;
 
+enum input_status {
+	INPUT_OK,
+	INPUT_SKIP,
+	INPUT_DELETE,
+	INPUT_STOP,
+	INPUT_CANCEL
+};
+
 struct menu_item {
 	int hotkey;
 	const char *text;
@@ -30,11 +38,13 @@ typedef enum input_status (*input_handler)(struct input *input, struct key *key)
 
 struct input {
 	input_handler handler;
+	bool allow_empty;
 	void *data;
 	char buf[SIZEOF_STR];
 };
 
-char *read_prompt_incremental(const char *prompt, bool edit_mode, input_handler handler, void *data);
+enum input_status prompt_default_handler(struct input *input, struct key *key);
+char *read_prompt_incremental(const char *prompt, bool edit_mode, bool allow_empty, input_handler handler, void *data);
 char *read_prompt(const char *prompt);
 void prompt_init(void);
 bool prompt_yesno(const char *prompt);
@@ -42,6 +52,7 @@ bool prompt_menu(const char *prompt, const struct menu_item *items, int *selecte
 
 enum request run_prompt_command(struct view *view, const char *argv[]);
 enum request open_prompt(struct view *view);
+enum request exec_run_request(struct view *view, struct run_request *req);
 
 #endif
 /* vim: set ts=8 sw=8 noexpandtab: */
