@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -13,51 +13,128 @@
 
 #include "tig/tig.h"
 #include "tig/argv.h"
+#include "tig/repo.h"
 #include "tig/options.h"
 #include "tig/prompt.h"
 
+static bool
+concat_argv(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *sep, bool quoted)
+{
+	size_t bufpos, argc;
+
+	for (bufpos = 0, argc = 0; argv[argc]; argc++) {
+		const char *arg_sep = argc ? sep : "";
+		const char *arg = argv[argc];
+		int pos;
+
+		if (quoted && arg[(pos = strcspn(arg, " \t\""))]) {
+			if (!string_nformat(buf, buflen, &bufpos, "%s\"", arg_sep))
+				return false;
+
+			while (*arg) {
+				int pos = strcspn(arg, "\"");
+				const char *qesc = arg[pos] == '"' ? "\\\"" : "";
+
+				if (!string_nformat(buf, buflen, &bufpos, "%.*s%s", pos, arg, qesc))
+					return false;
+				arg += pos + 1;
+			}
+
+			if (!string_nformat(buf, buflen, &bufpos, "\""))
+				return false;
+
+			continue;
+		}
+
+		if (!string_nformat(buf, buflen, &bufpos, "%s%s", arg_sep, arg))
+			return false;
+	}
+
+	return true;
+}
+
+bool
+argv_to_string_quoted(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *sep)
+{
+	return concat_argv(argv, buf, buflen, sep, true);
+}
+
 bool
 argv_to_string(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *sep)
 {
-	size_t bufpos, argc;
-
-	for (bufpos = 0, argc = 0; argv[argc]; argc++)
-		if (!string_nformat(buf, buflen, &bufpos, "%s%s",
-				argc ? sep : "", argv[argc]))
-			return FALSE;
-
-	return TRUE;
+	return concat_argv(argv, buf, buflen, sep, false);
 }
 
-static inline int
-get_arg_valuelen(const char *arg, char *quoted)
+static char *
+parse_arg(char **cmd, bool remove_quotes)
 {
-	if (*arg == '"' || *arg == '\'') {
-		const char *end = *arg == '"' ? "\"" : "'";
-		int valuelen = strcspn(arg + 1, end);
+	int quote = 0;
+	char *arg = *cmd;
+	char *next, *pos;
 
-		if (quoted)
-			*quoted = *arg;
-		return valuelen > 0 ? valuelen + 2 : strlen(arg);
-	} else {
-		if (quoted)
-			*quoted = 0;
-		return strcspn(arg, " \t");
+	for (pos = next = arg; *pos; pos++) {
+		int c = *pos;
+
+		if (c == '"' || c == '\'') {
+			if (quote == c) {
+				quote = 0;
+				if (remove_quotes) {
+					if (pos == arg) {
+						arg++;
+						next++;
+					}
+					continue;
+				}
+
+			} else if (!quote) {
+				quote = c;
+				if (remove_quotes) {
+					if (pos == arg) {
+						arg++;
+						next++;
+					}
+					continue;
+				}
+			}
+
+		} else if (quote && c == '\\') {
+			if (remove_quotes) {
+				if (pos == arg) {
+					arg++;
+					next++;
+				}
+			} else {
+				*next++ = *pos;
+			}
+			pos++;
+			if (!*pos)
+				break;
+		}
+
+		if (!quote && isspace(c))
+			break;
+
+		*next++ = *pos;
 	}
+
+	if (*pos)
+		*cmd = pos + 1;
+	else
+		*cmd = pos;
+	*next = 0;
+	return (!remove_quotes || !quote) ? arg : NULL;
 }
 
 static bool
 split_argv_string(const char *argv[SIZEOF_ARG], int *argc, char *cmd, bool remove_quotes)
 {
 	while (*cmd && *argc < SIZEOF_ARG) {
-		char quoted = 0;
-		int valuelen = get_arg_valuelen(cmd, &quoted);
-		bool advance = cmd[valuelen] != 0;
-		int quote_offset = !!(quoted && remove_quotes);
+		char *arg = parse_arg(&cmd, remove_quotes);
 
-		cmd[valuelen - quote_offset] = 0;
-		argv[(*argc)++] = chomp_string(cmd + quote_offset);
-		cmd = chomp_string(cmd + valuelen + advance);
+		if (!arg)
+			break;
+		argv[(*argc)++] = arg;
+		cmd = chomp_string(cmd);
 	}
 
 	if (*argc < SIZEOF_ARG)
@@ -68,13 +145,13 @@ split_argv_string(const char *argv[SIZEOF_ARG], int *argc, char *cmd, bool remov
 bool
 argv_from_string_no_quotes(const char *argv[SIZEOF_ARG], int *argc, char *cmd)
 {
-	return split_argv_string(argv, argc, cmd, TRUE);
+	return split_argv_string(argv, argc, cmd, true);
 }
 
 bool
 argv_from_string(const char *argv[SIZEOF_ARG], int *argc, char *cmd)
 {
-	return split_argv_string(argv, argc, cmd, FALSE);
+	return split_argv_string(argv, argc, cmd, false);
 }
 
 bool
@@ -118,8 +195,8 @@ argv_contains(const char **argv, const char *arg)
 
 	for (i = 0; argv && argv[i]; i++)
 		if (!strcmp(argv[i], arg))
-			return TRUE;
-	return FALSE;
+			return true;
+	return false;
 }
 
 DEFINE_ALLOCATOR(argv_realloc, const char *, SIZEOF_ARG)
@@ -131,10 +208,10 @@ argv_append(const char ***argv, const char *arg)
 	char *alloc;
 
 	if (!*arg && argc > 0)
-		return TRUE;
+		return true;
 
 	if (!argv_realloc(argv, argc, 2))
-		return FALSE;
+		return false;
 
 	alloc = strdup(arg);
 
@@ -151,33 +228,8 @@ argv_append_array(const char ***dst_argv, const char *src_argv[])
 
 	for (i = 0; src_argv && src_argv[i]; i++)
 		if (!argv_append(dst_argv, src_argv[i]))
-			return FALSE;
-	return TRUE;
-}
-
-bool
-argv_remove_quotes(const char *argv[])
-{
-	int argc;
-
-	for (argc = 0; argv[argc]; argc++) {
-		char quoted = 0;
-		const char *arg = argv[argc];
-		const int arglen = get_arg_valuelen(arg, &quoted);
-		const int unquotedlen = arglen - 1 - (arg[arglen - 1] == quoted);
-		char *unquoted;
-
-		if (!quoted)
-			continue;
-
-		unquoted = strndup(arg + 1, unquotedlen);
-		if (!unquoted)
-			return FALSE;
-		free((void *) arg);
-		argv[argc] = unquoted;
-	}
-
-	return TRUE;
+			return false;
+	return true;
 }
 
 bool
@@ -188,18 +240,21 @@ argv_copy(const char ***dst, const char *src[])
 	argv_free(*dst);
 	for (argc = 0; src[argc]; argc++)
 		if (!argv_append(dst, src[argc]))
-			return FALSE;
-	return TRUE;
+			return false;
+	return true;
 }
 
 /*
  * Argument formatting.
  */
 
+struct format_context;
+
 struct format_var {
 	const char *name;
 	size_t namelen;
-	const char *value;
+	bool (*formatter)(struct format_context *, struct format_var *);
+	void *value_ref;
 	const char *value_if_empty;
 };
 
@@ -211,7 +266,7 @@ struct format_context {
 	bool file_filter;
 };
 
-#define ARGV_ENV_INIT(name, ifempty, initval)	initval
+#define ARGV_ENV_INIT(type, name, ifempty, initval)	initval,
 
 struct argv_env argv_env = {
 	ARGV_ENV_INFO(ARGV_ENV_INIT)
@@ -228,7 +283,7 @@ format_expand_arg(struct format_context *format, const char *name, const char *e
 		char msgbuf[SIZEOF_STR];
 		const char *value;
 		const char *msgstart = name + STRING_SIZE("%(prompt");
-		int msglen = end - msgstart - 1;
+		const int msglen = end - msgstart - 1;
 
 		if (end && msglen > 0 && string_format(msgbuf, "%.*s", msglen, msgstart)) {
 			const char *msg = msgbuf;
@@ -241,27 +296,21 @@ format_expand_arg(struct format_context *format, const char *name, const char *e
 
 		value = read_prompt(prompt);
 		if (value == NULL)
-			return FALSE;
+			return false;
 		return string_format_from(format->buf, &format->bufpos, "%s", value);
 	}
 
 	for (i = 0; i < format->vars_size; i++) {
-		const char *value;
-
-		if (strncmp(name, vars[i].name, vars[i].namelen))
+		if (string_enum_compare(name, vars[i].name, vars[i].namelen))
 			continue;
 
-		if (vars[i].value == argv_env.file && !format->file_filter)
-			return TRUE;
+		if (vars[i].value_ref == &argv_env.file && !format->file_filter)
+			return true;
 
-		value = *vars[i].value ? vars[i].value : vars[i].value_if_empty;
-		if (!*value)
-			return TRUE;
-
-		return string_format_from(format->buf, &format->bufpos, "%s", value);
+		return vars[i].formatter(format, &vars[i]);
 	}
 
-	return FALSE;
+	return false;
 }
 
 static bool
@@ -271,15 +320,19 @@ format_append_arg(struct format_context *format, const char ***dst_argv, const c
 	format->bufpos = 0;
 
 	while (arg) {
-		char *var = strstr(arg, "%(");
-		int len = var ? var - arg : strlen(arg);
-		char *next = var ? strchr(var, ')') + 1 : NULL;
+		const char *var = strstr(arg, "%(");
+		const char *closing = var ? strchr(var, ')') : NULL;
+		const char *next = closing ? closing + 1 : NULL;
+		const int len = var ? var - arg : strlen(arg);
+
+		if (var && !closing)
+			return false;
 
 		if (len && !string_format_from(format->buf, &format->bufpos, "%.*s", len, arg))
-			return FALSE;
+			return false;
 
 		if (var && !format_expand_arg(format, var, next))
-			return FALSE;
+			return false;
 
 		arg = next;
 	}
@@ -293,22 +346,74 @@ format_append_argv(struct format_context *format, const char ***dst_argv, const
 	int argc;
 
 	if (!src_argv)
-		return TRUE;
+		return true;
 
 	for (argc = 0; src_argv[argc]; argc++)
 		if (!format_append_arg(format, dst_argv, src_argv[argc]))
-			return FALSE;
+			return false;
 
 	return src_argv[argc] == NULL;
 }
 
+static bool
+argv_string_formatter(struct format_context *format, struct format_var *var)
+{
+	argv_string *value_ref = var->value_ref;
+	const char *value = *value_ref;
+
+	if (!*value)
+		value = var->value_if_empty;
+
+	if (!*value)
+		return true;
+
+	return string_format_from(format->buf, &format->bufpos, "%s", value);
+}
+
+static bool
+argv_number_formatter(struct format_context *format, struct format_var *var)
+{
+	unsigned long value = *(unsigned long *) var->value_ref;
+
+	return string_format_from(format->buf, &format->bufpos, "%ld", value);
+}
+
+static bool
+bool_formatter(struct format_context *format, struct format_var *var)
+{
+	bool value = *(bool *)var->value_ref;
+
+	return string_format_from(format->buf, &format->bufpos, "%s", value ? "true" : "false");
+}
+
+static bool
+repo_str_formatter(struct format_context *format, struct format_var *var)
+{
+	return argv_string_formatter(format, var);
+}
+
+static bool
+repo_ref_formatter(struct format_context *format, struct format_var *var)
+{
+	return argv_string_formatter(format, var);
+}
+
+static bool
+repo_rev_formatter(struct format_context *format, struct format_var *var)
+{
+	return argv_string_formatter(format, var);
+}
+
 bool
 argv_format(struct argv_env *argv_env, const char ***dst_argv, const char *src_argv[], bool first, bool file_filter)
 {
 	struct format_var vars[] = {
-#define FORMAT_VAR(name, ifempty, initval) \
-	{ "%(" #name ")", STRING_SIZE("%(" #name ")"), argv_env->name, ifempty }
+#define FORMAT_VAR(type, name, ifempty, initval) \
+	{ "%(" #name ")", STRING_SIZE("%(" #name ")"), type ## _formatter, &argv_env->name, ifempty },
 		ARGV_ENV_INFO(FORMAT_VAR)
+#define FORMAT_REPO_VAR(type, name) \
+	{ "%(repo:" #name ")", STRING_SIZE("%(repo:" #name ")"), type ## _formatter, &repo.name, "" },
+		REPO_INFO(FORMAT_REPO_VAR)
 	};
 	struct format_context format = { vars, ARRAY_SIZE(vars), "", 0, file_filter };
 	int argc;
@@ -319,7 +424,7 @@ argv_format(struct argv_env *argv_env, const char ***dst_argv, const char *src_a
 		const char *arg = src_argv[argc];
 
 		if (!strcmp(arg, "%(fileargs)")) {
-			if (file_filter && !argv_append_array(dst_argv, opt_file_argv))
+			if (file_filter && !argv_append_array(dst_argv, opt_file_args))
 				break;
 
 		} else if (!strcmp(arg, "%(diffargs)")) {
@@ -330,13 +435,21 @@ argv_format(struct argv_env *argv_env, const char ***dst_argv, const char *src_a
 			if (!format_append_argv(&format, dst_argv, opt_blame_options))
 				break;
 
+		} else if (!strcmp(arg, "%(logargs)")) {
+			if (!format_append_argv(&format, dst_argv, opt_log_options))
+				break;
+
+		} else if (!strcmp(arg, "%(mainargs)")) {
+			if (!format_append_argv(&format, dst_argv, opt_main_options))
+				break;
+
 		} else if (!strcmp(arg, "%(cmdlineargs)")) {
-			if (!format_append_argv(&format, dst_argv, opt_cmdline_argv))
+			if (!format_append_argv(&format, dst_argv, opt_cmdline_args))
 				break;
 
 		} else if (!strcmp(arg, "%(revargs)") ||
 			   (first && !strcmp(arg, "%(commit)"))) {
-			if (!argv_append_array(dst_argv, opt_rev_argv))
+			if (!argv_append_array(dst_argv, opt_rev_args))
 				break;
 
 		} else if (!format_append_arg(&format, dst_argv, arg)) {
@@ -366,14 +479,14 @@ argv_find_rev_flag(const char *argv[], size_t argc, const char *arg, size_t argl
 			continue;
 
 		if (with_graph)
-			*with_graph = FALSE;
+			*with_graph = false;
 		if (with_reflog)
-			*with_reflog = TRUE;
+			*with_reflog = true;
 
-		return TRUE;
+		return true;
 	}
 
-	return FALSE;
+	return false;
 }
 
 bool
@@ -449,8 +562,8 @@ argv_parse_rev_flag(const char *arg, struct rev_flags *rev_flags)
 		"-S",
 	};
 	size_t arglen = strlen(arg);
-	bool graph = TRUE;
-	bool reflog = FALSE;
+	bool graph = true;
+	bool reflog = false;
 	size_t search = 0;
 
 	if (argv_find_rev_flag(with_graph, ARRAY_SIZE(with_graph), arg, arglen, NULL, NULL, NULL) ||
@@ -462,10 +575,10 @@ argv_parse_rev_flag(const char *arg, struct rev_flags *rev_flags)
 			rev_flags->with_graph = graph;
 			rev_flags->with_reflog = reflog;
 		}
-		return TRUE;
+		return true;
 	}
 
-	return FALSE;
+	return false;
 }
 
 char *
@@ -475,7 +588,7 @@ argv_format_arg(struct argv_env *argv_env, const char *src_arg)
 	const char **dst_argv = NULL;
 	char *dst_arg = NULL;
 
-	if (argv_format(argv_env, &dst_argv, src_argv, FALSE, TRUE))
+	if (argv_format(argv_env, &dst_argv, src_argv, false, true))
 		dst_arg = (char *) dst_argv[0];
 
 	free(dst_argv);
