@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,14 +31,41 @@
 
 DEFINE_ALLOCATOR(realloc_reflogs, char *, 32)
 
+bool
+main_status_exists(struct view *view, enum line_type type)
+{
+	struct main_state *state;
+
+	refresh_view(view);
+
+	state = view->private;
+	state->goto_line_type = type;
+	if (type == LINE_STAT_STAGED && state->add_changes_staged)
+		return true;
+	if (type == LINE_STAT_UNSTAGED && state->add_changes_unstaged)
+		return true;
+
+	return false;
+}
+
+static bool main_add_changes(struct view *view, struct main_state *state, const char *parent);
+
 static void
 main_register_commit(struct view *view, struct commit *commit, const char *ids, bool is_boundary)
 {
 	struct main_state *state = view->private;
+	struct graph *graph = state->graph;
 
 	string_copy_rev(commit->id, ids);
+
+	/* FIXME: lazily check index state here instead of in main_open. */
+	if ((state->add_changes_unstaged || state->add_changes_staged) && is_head_commit(commit->id)) {
+		main_add_changes(view, state, ids);
+		state->add_changes_unstaged = state->add_changes_staged = false;
+	}
+
 	if (state->with_graph)
-		graph_add_commit(&state->graph, &commit->graph, commit->id, ids, is_boundary);
+		graph->add_commit(graph, &commit->graph, commit->id, ids, is_boundary);
 }
 
 static struct commit *
@@ -53,7 +80,7 @@ main_add_commit(struct view *view, enum line_type type, struct commit *template,
 
 	/* FIXME: More graceful handling of titles; append "..." to
 	 * shortened titles, etc. */
-	string_expand(buf, sizeof(buf), title, 1);
+	string_expand(buf, sizeof(buf), title, strlen(title), 1);
 	title = buf;
 	titlelen = strlen(title);
 
@@ -63,7 +90,6 @@ main_add_commit(struct view *view, enum line_type type, struct commit *template,
 
 	*commit = *template;
 	strncpy(commit->title, title, titlelen);
-	state->graph.canvas = &commit->graph;
 	memset(template, 0, sizeof(*template));
 	state->reflogmsg[0] = 0;
 
@@ -75,7 +101,7 @@ static inline void
 main_flush_commit(struct view *view, struct commit *commit)
 {
 	if (*commit->id)
-		main_add_commit(view, LINE_MAIN_COMMIT, commit, "", FALSE);
+		main_add_commit(view, LINE_MAIN_COMMIT, commit, "", false);
 }
 
 static bool
@@ -83,96 +109,120 @@ main_add_changes_commit(struct view *view, enum line_type type, const char *pare
 {
 	char ids[SIZEOF_STR] = NULL_ID " ";
 	struct main_state *state = view->private;
-	struct commit commit = {};
+	struct graph *graph = state->graph;
+	struct commit commit = {{0}};
 	struct timeval now;
 	struct timezone tz;
 
 	if (!parent)
-		return TRUE;
+		return true;
 
 	if (*parent)
 		string_copy_rev(ids + STRING_SIZE(NULL_ID " "), parent);
 	else
 		ids[STRING_SIZE(NULL_ID)] = 0;
 
-	if (!gettimeofday(&now, &tz)) {
+	if (!time_now(&now, &tz)) {
 		commit.time.tz = tz.tz_minuteswest * 60;
 		commit.time.sec = now.tv_sec - commit.time.tz;
 	}
 
 	commit.author = &unknown_ident;
-	main_register_commit(view, &commit, ids, FALSE);
-	if (!main_add_commit(view, type, &commit, title, TRUE))
-		return FALSE;
+	main_register_commit(view, &commit, ids, false);
+	if (state->with_graph && *parent)
+		graph->render_parents(graph, &commit.graph);
 
-	if (state->with_graph) {
-		if (*parent)
-			return graph_render_parents(&state->graph);
-		state->add_changes_parents = TRUE;
-	}
+	if (!main_add_commit(view, type, &commit, title, true))
+		return false;
 
-	return TRUE;
+	if (state->goto_line_type == type)
+		select_view_line(view, view->lines - 1);
+
+	return true;
 }
 
 static bool
-main_add_changes_commits(struct view *view, struct main_state *state, const char *parent)
+main_check_index(struct view *view, struct main_state *state)
 {
-	const char *staged_parent = NULL_ID;
-	const char *unstaged_parent = parent;
 	struct index_diff diff;
 
-	if (!index_diff(&diff, FALSE, FALSE))
-		return FALSE;
+	if (!index_diff(&diff, false, false))
+		return false;
 
 	if (!diff.unstaged) {
-		unstaged_parent = NULL;
-		staged_parent = parent;
 		watch_apply(&view->watch, WATCH_INDEX_UNSTAGED_NO);
 	} else {
 		watch_apply(&view->watch, WATCH_INDEX_UNSTAGED_YES);
+		state->add_changes_unstaged = true;
 	}
 
 	if (!diff.staged) {
-		staged_parent = NULL;
 		watch_apply(&view->watch, WATCH_INDEX_STAGED_NO);
 	} else {
 		watch_apply(&view->watch, WATCH_INDEX_STAGED_YES);
+		state->add_changes_staged = true;
 	}
 
-	return main_add_changes_commit(view, LINE_STAT_STAGED, staged_parent, "Staged changes")
-	    && main_add_changes_commit(view, LINE_STAT_UNSTAGED, unstaged_parent, "Unstaged changes");
+	return true;
+}
+
+static bool
+main_add_changes(struct view *view, struct main_state *state, const char *parent)
+{
+	const char *staged_parent = parent;
+	const char *unstaged_parent = NULL_ID;
+
+	if (!state->add_changes_staged) {
+		staged_parent = NULL;
+		unstaged_parent = parent;
+	}
+
+	if (!state->add_changes_unstaged) {
+		unstaged_parent = NULL;
+	}
+
+	return main_add_changes_commit(view, LINE_STAT_UNSTAGED, unstaged_parent, "Unstaged changes")
+	    && main_add_changes_commit(view, LINE_STAT_STAGED, staged_parent, "Staged changes");
 }
 
 static bool
 main_check_argv(struct view *view, const char *argv[])
 {
 	struct main_state *state = view->private;
-	bool with_reflog = FALSE;
+	bool with_reflog = false;
 	int i;
 
 	for (i = 0; argv[i]; i++) {
 		const char *arg = argv[i];
-		struct rev_flags rev_flags = {};
+		struct rev_flags rev_flags = {0};
 
 		if (!strcmp(arg, "--graph")) {
 			struct view_column *column = get_view_column(view, VIEW_COLUMN_COMMIT_TITLE);
 
 			if (column) {
-				column->opt.commit_title.graph = TRUE;
+				column->opt.commit_title.graph = true;
 				if (opt_commit_order != COMMIT_ORDER_REVERSE)
-					state->with_graph = TRUE;
+					state->with_graph = true;
 			}
 			argv[i] = "";
 			continue;
 		}
 
+		if (!strcmp(arg, "--merge")) {
+			argv_append(&opt_rev_args, "--boundary");
+			continue;
+		}
+
+		if (!strcmp(arg, "--first-parent"))
+			state->first_parent = true;
+
 		if (!argv_parse_rev_flag(arg, &rev_flags))
 			continue;
 
 		if (rev_flags.with_reflog)
-			with_reflog = TRUE;
+			with_reflog = true;
 		if (!rev_flags.with_graph)
-			state->with_graph = FALSE;
+			state->with_graph = false;
 		arg += rev_flags.search_offset;
 		if (*arg && !*view->env->search)
 			string_ncopy(view->env->search, arg, strlen(arg));
@@ -181,38 +231,52 @@ main_check_argv(struct view *view, const char *argv[])
 	return with_reflog;
 }
 
+static enum graph_display
+main_with_graph(struct view *view, struct view_column *column, enum open_flags flags)
+{
+	return column && opt_commit_order != COMMIT_ORDER_REVERSE && !open_in_pager_mode(flags)
+	       ? column->opt.commit_title.graph : GRAPH_DISPLAY_NO;
+}
+
 static bool
 main_open(struct view *view, enum open_flags flags)
 {
+	struct view_column *commit_title_column = get_view_column(view, VIEW_COLUMN_COMMIT_TITLE);
+	enum graph_display graph_display = main_with_graph(view, commit_title_column, flags);
 	const char *pretty_custom_argv[] = {
-		GIT_MAIN_LOG_CUSTOM(encoding_arg, commit_order_arg(), "%(cmdlineargs)", "%(revargs)", "%(fileargs)")
+		GIT_MAIN_LOG(encoding_arg, commit_order_arg_with_graph(graph_display),
+			"%(mainargs)", "%(cmdlineargs)", "%(revargs)", "%(fileargs)",
+			log_custom_pretty_arg())
 	};
 	const char *pretty_raw_argv[] = {
-		GIT_MAIN_LOG_RAW(encoding_arg, commit_order_arg(), "%(cmdlineargs)", "%(revargs)", "%(fileargs)")
+		GIT_MAIN_LOG_RAW(encoding_arg, commit_order_arg_with_graph(graph_display),
+			"%(mainargs)", "%(cmdlineargs)", "%(revargs)", "%(fileargs)")
 	};
 	struct main_state *state = view->private;
 	const char **main_argv = pretty_custom_argv;
-	struct view_column *column;
 	enum watch_trigger changes_triggers = WATCH_NONE;
 
 	if (opt_show_changes && repo.is_inside_work_tree)
 		changes_triggers |= WATCH_INDEX;
 
-	column = get_view_column(view, VIEW_COLUMN_COMMIT_TITLE);
-	state->with_graph = column && column->opt.commit_title.graph &&
-			    opt_commit_order != COMMIT_ORDER_REVERSE;
+	state->with_graph = graph_display != GRAPH_DISPLAY_NO;
 
-	if (opt_rev_argv && main_check_argv(view, opt_rev_argv))
+	if (opt_rev_args && main_check_argv(view, opt_rev_args))
 		main_argv = pretty_raw_argv;
 
+	if (state->with_graph) {
+		state->graph = init_graph(commit_title_column->opt.commit_title.graph);
+		if (!state->graph)
+			return false;
+	}
+
 	if (open_in_pager_mode(flags)) {
 		changes_triggers = WATCH_NONE;
-		state->with_graph = FALSE;
 	}
 
 	/* This calls reset_view() so must be before adding changes commits. */
 	if (!begin_update(view, NULL, main_argv, flags))
-		return FALSE;
+		return false;
 
 	/* Register watch before changes commits are added to record the
 	 * start. */
@@ -220,9 +284,9 @@ main_open(struct view *view, enum open_flags flags)
 		watch_register(&view->watch, WATCH_HEAD | WATCH_REFS | changes_triggers);
 
 	if (changes_triggers)
-		main_add_changes_commits(view, state, "");
+		main_check_index(view, state);
 
-	return TRUE;
+	return true;
 }
 
 void
@@ -237,6 +301,9 @@ main_done(struct view *view)
 		free(commit->graph.symbols);
 	}
 
+	if (state->graph)
+		state->graph->done(state->graph);
+
 	for (i = 0; i < state->reflogs; i++)
 		free(state->reflog[i]);
 	free(state->reflog);
@@ -245,10 +312,10 @@ main_done(struct view *view)
 #define main_check_commit_refs(line)	!((line)->no_commit_refs)
 #define main_mark_no_commit_refs(line)	(((struct line *) (line))->no_commit_refs = 1)
 
-static inline struct ref_list *
+static inline const struct ref *
 main_get_commit_refs(const struct line *line, struct commit *commit)
 {
-	struct ref_list *refs = NULL;
+	const struct ref *refs = NULL;
 
 	if (main_check_commit_refs(line) && !(refs = get_ref_list(commit->id)))
 		main_mark_no_commit_refs(line);
@@ -261,7 +328,6 @@ main_get_column_data(struct view *view, const struct line *line, struct view_col
 {
 	struct main_state *state = view->private;
 	struct commit *commit = line->data;
-	struct ref_list *refs = NULL;
 
 	column_data->author = commit->author;
 	column_data->date = &commit->time;
@@ -270,13 +336,14 @@ main_get_column_data(struct view *view, const struct line *line, struct view_col
 		column_data->reflog = state->reflog[line->lineno - 1];
 
 	column_data->commit_title = commit->title;
-	if (state->with_graph)
-		column_data->graph = &commit->graph;
+	if (state->with_graph) {
+		column_data->graph = state->graph;
+		column_data->graph_canvas = &commit->graph;
+	}
 
-	if ((refs = main_get_commit_refs(line, commit)))
-		column_data->refs = refs;
+	column_data->refs = main_get_commit_refs(line, commit);
 
-	return TRUE;
+	return true;
 }
 
 static bool
@@ -286,12 +353,12 @@ main_add_reflog(struct view *view, struct main_state *state, char *reflog)
 	int id_width;
 
 	if (!end)
-		return FALSE;
+		return false;
 	*end = 0;
 
 	if (!realloc_reflogs(&state->reflog, state->reflogs, 1)
 	    || !(reflog = strdup(reflog)))
-		return FALSE;
+		return false;
 
 	state->reflog[state->reflogs++] = reflog;
 	id_width = strlen(reflog);
@@ -300,18 +367,18 @@ main_add_reflog(struct view *view, struct main_state *state, char *reflog)
 
 		state->reflog_width = id_width;
 		if (column && column->opt.id.display)
-			view->force_redraw = TRUE;
+			view->force_redraw = true;
 	}
 
-	return TRUE;
+	return true;
 }
 
 /* Reads git log --pretty=raw output and parses it into the commit struct. */
 bool
-main_read(struct view *view, struct buffer *buf)
+main_read(struct view *view, struct buffer *buf, bool force_stop)
 {
 	struct main_state *state = view->private;
-	struct graph *graph = &state->graph;
+	struct graph *graph = state->graph;
 	enum line_type type;
 	struct commit *commit = &state->current;
 	char *line;
@@ -319,7 +386,7 @@ main_read(struct view *view, struct buffer *buf)
 	if (!buf) {
 		main_flush_commit(view, commit);
 
-		if (failed_to_load_initial_view(view))
+		if (!force_stop && failed_to_load_initial_view(view))
 			die("No revisions match the given arguments.");
 		if (view->lines > 0) {
 			struct commit *last = view->line[view->lines - 1].data;
@@ -331,9 +398,9 @@ main_read(struct view *view, struct buffer *buf)
 			}
 		}
 
-		if (state->with_graph)
-			done_graph(graph);
-		return TRUE;
+		if (state->graph)
+			state->graph->done_rendering(graph);
+		return true;
 	}
 
 	line = buf->data;
@@ -342,48 +409,50 @@ main_read(struct view *view, struct buffer *buf)
 		bool is_boundary;
 		char *author;
 
-		state->in_header = TRUE;
+		state->in_header = true;
 		line += STRING_SIZE("commit ");
 		is_boundary = *line == '-';
 		while (*line && !isalnum(*line))
 			line++;
 
-		if (state->add_changes_parents) {
-			state->add_changes_parents = FALSE;
-			if (!graph_add_parent(graph, line))
-				return FALSE;
-			graph->has_parents = TRUE;
-			graph_render_parents(graph);
-		}
-
 		main_flush_commit(view, commit);
-		main_register_commit(view, &state->current, line, is_boundary);
 
 		author = io_memchr(buf, line, 0);
+
+		if (state->first_parent) {
+			char *parent = strchr(line, ' ');
+			char *parent_end = parent ? strchr(parent + 1, ' ') : NULL;
+
+			if (parent_end)
+				*parent_end = 0;
+		}
+
+		main_register_commit(view, &state->current, line, is_boundary);
+
 		if (author) {
 			char *title = io_memchr(buf, author, 0);
 
 			parse_author_line(author, &commit->author, &commit->time);
 			if (state->with_graph)
-				graph_render_parents(graph);
+				graph->render_parents(graph, &commit->graph);
 			if (title)
-				main_add_commit(view, LINE_MAIN_COMMIT, commit, title, FALSE);
+				main_add_commit(view, LINE_MAIN_COMMIT, commit, title, false);
 		}
 
-		return TRUE;
+		return true;
 	}
 
 	if (!*commit->id)
-		return TRUE;
+		return true;
 
 	/* Empty line separates the commit header from the log itself. */
 	if (*line == '\0')
-		state->in_header = FALSE;
+		state->in_header = false;
 
 	switch (type) {
 	case LINE_PP_REFLOG:
 		if (!main_add_reflog(view, state, line + STRING_SIZE("Reflog: ")))
-			return FALSE;
+			return false;
 		break;
 
 	case LINE_PP_REFLOGMSG:
@@ -392,15 +461,15 @@ main_read(struct view *view, struct buffer *buf)
 		break;
 
 	case LINE_PARENT:
-		if (state->with_graph && !graph->has_parents)
-			graph_add_parent(graph, line + STRING_SIZE("parent "));
+		if (state->with_graph)
+			graph->add_parent(graph, line + STRING_SIZE("parent "));
 		break;
 
 	case LINE_AUTHOR:
 		parse_author_line(line + STRING_SIZE("author "),
 				  &commit->author, &commit->time);
 		if (state->with_graph)
-			graph_render_parents(graph);
+			graph->render_parents(graph, &commit->graph);
 		break;
 
 	default:
@@ -425,10 +494,10 @@ main_read(struct view *view, struct buffer *buf)
 			break;
 		if (*state->reflogmsg)
 			line = state->reflogmsg;
-		main_add_commit(view, LINE_MAIN_COMMIT, commit, line, FALSE);
+		main_add_commit(view, LINE_MAIN_COMMIT, commit, line, false);
 	}
 
-	return TRUE;
+	return true;
 }
 
 enum request
@@ -449,7 +518,7 @@ main_request(struct view *view, enum request request, struct line *line)
 	case REQ_VIEW_DIFF:
 	case REQ_ENTER:
 		if (view_is_displayed(view) && display[0] != view)
-			maximize_view(view, TRUE);
+			maximize_view(view, true);
 
 		if (line->type == LINE_STAT_UNSTAGED
 		    || line->type == LINE_STAT_STAGED)
@@ -459,10 +528,14 @@ main_request(struct view *view, enum request request, struct line *line)
 		break;
 
 	case REQ_REFRESH:
-		load_refs(TRUE);
+		load_refs(true);
 		refresh_view(view);
 		break;
 
+	case REQ_PARENT:
+		goto_id(view, "%(commit)^", true, false);
+		break;
+
 	default:
 		return request;
 	}
@@ -470,16 +543,6 @@ main_request(struct view *view, enum request request, struct line *line)
 	return REQ_NONE;
 }
 
-static void
-main_update_env(struct view *view, struct line *line, struct commit *commit)
-{
-	struct ref_list *list = main_get_commit_refs(line, commit);
-	size_t i;
-
-	for (i = 0; list && i < list->size; i++)
-		ref_update_env(view->env, list->refs[list->size - i - 1], !i);
-}
-
 void
 main_select(struct view *view, struct line *line)
 {
@@ -489,8 +552,11 @@ main_select(struct view *view, struct line *line)
 		string_ncopy(view->ref, commit->title, strlen(commit->title));
 		status_stage_info(view->env->status, line->type, NULL);
 	} else {
+		const struct ref *ref = main_get_commit_refs(line, commit);
+
 		string_copy_rev(view->ref, commit->id);
-		main_update_env(view, line, commit);
+		if (ref)
+			ref_update_env(view->env, ref, true);
 	}
 	string_copy_rev(view->env->commit, commit->id);
 }
