@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,7 +28,7 @@
  */
 
 #define DEFINE_OPTION_VARIABLES(name, type, flags) type opt_##name;
-OPTION_INFO(DEFINE_OPTION_VARIABLES);
+OPTION_INFO(DEFINE_OPTION_VARIABLES)
 
 static struct option_info option_info[] = {
 #define DEFINE_OPTION_INFO(name, type, flags) { #name, STRING_SIZE(#name), #type, &opt_##name },
@@ -36,15 +36,32 @@ static struct option_info option_info[] = {
 };
 
 struct option_info *
-find_option_info(struct option_info *option, size_t options, const char *name)
+find_option_info(struct option_info *option, size_t options, const char *prefix, const char *name)
 {
 	size_t namelen = strlen(name);
+	char prefixed[SIZEOF_STR];
 	int i;
 
-	for (i = 0; i < options; i++)
+	if (*prefix && namelen == strlen(prefix) &&
+	    !string_enum_compare(prefix, name, namelen)) {
+		name = "display";
+		namelen = strlen(name);
+	}
+
+	for (i = 0; i < options; i++) {
+		if (!strcmp(option[i].type, "view_settings") &&
+		    enum_equals_prefix(option[i], name, namelen))
+			return &option[i];
+
 		if (enum_equals(option[i], name, namelen))
 			return &option[i];
 
+		if (enum_name_prefixed(prefixed, sizeof(prefixed), prefix, option[i].name) &&
+		    namelen == strlen(prefixed) &&
+		    !string_enum_compare(prefixed, name, namelen))
+			return &option[i];
+	}
+
 	return NULL;
 }
 
@@ -66,7 +83,36 @@ mark_option_seen(void *value)
 	struct option_info *option = find_option_info_by_value(value);
 
 	if (option)
-		option->seen = TRUE;
+		option->seen = true;
+}
+
+struct option_info *
+find_column_option_info(enum view_column_type type, union view_column_options *opts,
+			const char *option, struct option_info *column_info,
+			const char **column_name)
+{
+#define DEFINE_COLUMN_OPTION_INFO(name, type, flags) \
+	{ #name, STRING_SIZE(#name), #type, &opt->name, flags },
+
+#define DEFINE_COLUMN_OPTION_INFO_CHECK(name, id, options) \
+	if (type == VIEW_COLUMN_##id) { \
+		struct name##_options *opt = &opts->name; \
+		struct option_info info[] = { \
+			options(DEFINE_COLUMN_OPTION_INFO) \
+		}; \
+		struct option_info *match; \
+		match = find_option_info(info, ARRAY_SIZE(info), #name, option); \
+		if (match) { \
+			*column_info = *match; \
+			*column_name = #name; \
+			return column_info; \
+		} \
+	}
+
+	COLUMN_OPTIONS(DEFINE_COLUMN_OPTION_INFO_CHECK);
+
+	*column_name = NULL;
+	return NULL;
 }
 
 /*
@@ -75,12 +121,7 @@ mark_option_seen(void *value)
 
 iconv_t opt_iconv_out		= ICONV_NONE;
 char opt_editor[SIZEOF_STR]	= "";
-const char **opt_cmdline_argv	= NULL;
-const char **opt_rev_argv	= NULL;
-const char **opt_file_argv	= NULL;
-char opt_env_lines[64]		= "";
-char opt_env_columns[64]	= "";
-char *opt_env[]			= { opt_env_lines, opt_env_columns, NULL };
+const char **opt_cmdline_args	= NULL;
 
 /*
  * Mapping between options and command argument mapping.
@@ -98,6 +139,19 @@ diff_context_arg()
 	return opt_diff_context_arg;
 }
 
+const char *
+use_mailmap_arg()
+{
+	return opt_mailmap ? "--use-mailmap" : "";
+}
+
+const char *
+log_custom_pretty_arg(void)
+{
+	return opt_mailmap
+		? "--pretty=format:commit %m %H %P%x00%aN <%aE> %ad%x00%s"
+		: "--pretty=format:commit %m %H %P%x00%an <%ae> %ad%x00%s";
+}
 
 #define ENUM_ARG(enum_name, arg_string) ENUM_MAP_ENTRY(arg_string, enum_name)
 
@@ -115,6 +169,7 @@ ignore_space_arg()
 }
 
 static const struct enum_map_entry commit_order_arg_map[] = {
+	ENUM_ARG(COMMIT_ORDER_AUTO,		""),
 	ENUM_ARG(COMMIT_ORDER_DEFAULT,		""),
 	ENUM_ARG(COMMIT_ORDER_TOPO,		"--topo-order"),
 	ENUM_ARG(COMMIT_ORDER_DATE,		"--date-order"),
@@ -128,6 +183,18 @@ commit_order_arg()
 	return commit_order_arg_map[opt_commit_order].name;
 }
 
+const char *
+commit_order_arg_with_graph(enum graph_display graph_display)
+{
+	enum commit_order commit_order = opt_commit_order;
+
+	if (commit_order == COMMIT_ORDER_AUTO &&
+	    graph_display != GRAPH_DISPLAY_NO)
+		commit_order = COMMIT_ORDER_TOPO;
+
+	return commit_order_arg_map[commit_order].name;
+}
+
 /* Use --show-notes to support Git >= 1.7.6 */
 #define NOTES_ARG	"--show-notes"
 #define NOTES_EQ_ARG	NOTES_ARG "="
@@ -165,14 +232,14 @@ update_options_from_argv(const char *argv[])
 		}
 
 		if (!strcmp(flag, "--no-notes")) {
-			opt_show_notes = FALSE;
+			opt_show_notes = false;
 			mark_option_seen(&opt_show_notes);
 			continue;
 		}
 
 		if (!prefixcmp(flag, "--show-notes") ||
 		    !prefixcmp(flag, "--notes")) {
-			opt_show_notes = TRUE;
+			opt_show_notes = true;
 			string_ncopy(opt_notes_arg, flag, strlen(flag));
 			mark_option_seen(&opt_show_notes);
 			continue;
@@ -224,12 +291,17 @@ static const struct enum_map_entry attr_map[] = {
 enum status_code
 parse_step(double *opt, const char *arg)
 {
-	*opt = atoi(arg);
+	int value = atoi(arg);
+
+	if (!value && !isdigit(*arg))
+		return error("Invalid double or percentage");
+
+	*opt = value;
 	if (!strchr(arg, '%'))
 		return SUCCESS;
 
 	/* "Shift down" so 100% and 1 does not conflict. */
-	*opt = (*opt - 1) / 100;
+	*opt /= 100;
 	if (*opt >= 1.0) {
 		*opt = 0.99;
 		return error("Percentage is larger than 100%%");
@@ -258,11 +330,13 @@ static bool
 set_color(int *color, const char *name)
 {
 	if (map_enum(color, color_map, name))
-		return TRUE;
+		return true;
+	/* Git expects a plain int w/o prefix, however, color<int> is
+	 * the preferred Tig color notation.  */
 	if (!prefixcmp(name, "color"))
-		return parse_int(color, name + 5, 0, 255) == SUCCESS;
-	/* Used when reading git colors. Git expects a plain int w/o prefix.  */
-	return parse_int(color, name, 0, 255) == SUCCESS;
+		name += 5;
+	return string_isnumber(name) &&
+	       parse_int(color, name, 0, 255) == SUCCESS;
 }
 
 #define is_quoted(c)	((c) == '"' || (c) == '\'')
@@ -316,7 +390,7 @@ find_remapped(const char *remapped[][2], size_t remapped_size, const char *arg)
 static enum status_code
 option_color_command(int argc, const char *argv[])
 {
-	struct line_rule rule = {};
+	struct line_rule rule = {0};
 	const char *prefix = NULL;
 	struct line_info *info;
 	enum status_code code;
@@ -341,6 +415,7 @@ option_color_command(int argc, const char *argv[])
 			{ "diff-tree",			"'diff-tree '" },
 			{ "filename",			"file" },
 			{ "help-keymap",		"help.section" },
+			{ "main-revgraph",		"" },
 			{ "pp-adate",			"'AuthorDate: '" },
 			{ "pp-author",			"'Author: '" },
 			{ "pp-cdate",			"'CommitDate: '" },
@@ -359,6 +434,8 @@ option_color_command(int argc, const char *argv[])
 
 		index = find_remapped(obsolete, ARRAY_SIZE(obsolete), rule.name);
 		if (index != -1) {
+			if (!*obsolete[index][1])
+				return error("%s is obsolete", argv[0]);
 			/* Keep the initial prefix if defined. */
 			code = parse_color_name(obsolete[index][1], &rule, prefix ? NULL : &prefix);
 			if (code != SUCCESS)
@@ -395,25 +472,43 @@ static enum status_code
 parse_bool(bool *opt, const char *arg)
 {
 	*opt = (!strcmp(arg, "1") || !strcmp(arg, "true") || !strcmp(arg, "yes"))
-		? TRUE : FALSE;
+		? true : false;
 	if (*opt || !strcmp(arg, "0") || !strcmp(arg, "false") || !strcmp(arg, "no"))
 		return SUCCESS;
 	return error("Non-boolean value treated as false: %s", arg);
 }
 
 static enum status_code
-parse_enum(unsigned int *opt, const char *arg, const struct enum_map *map)
+parse_enum(const char *name, unsigned int *opt, const char *arg,
+	   const struct enum_map *map)
 {
 	bool is_true;
+	enum status_code code;
 
 	assert(map->size > 1);
 
 	if (map_enum_do(map->entries, map->size, (int *) opt, arg))
 		return SUCCESS;
 
-	parse_bool(&is_true, arg);
+	code = parse_bool(&is_true, arg);
 	*opt = is_true ? map->entries[1].value : map->entries[0].value;
-	return SUCCESS;
+	if (code == SUCCESS)
+		return code;
+
+	if (!strcmp(name, "date-display")) {
+		const char *msg = "";
+
+		if (!strcasecmp(arg, "local"))
+			msg = ", use the 'date-local' column option";
+		else if (!strcasecmp(arg, "short"))
+			msg = ", use the 'custom' display mode and set 'date-format'";
+
+		*opt = map->entries[1].value;
+		return error("'%s' is no longer supported for %s%s", arg, name, msg);
+	}
+
+	return error("'%s' is not a valid value for %s; using %s",
+		     arg, name, enum_name(map->entries[*opt].name));
 }
 
 static enum status_code
@@ -475,7 +570,7 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 		if (parse_bool(option->value, arg) == SUCCESS)
 			return SUCCESS;
 
-		*value = TRUE;
+		*value = true;
 		string_copy(opt_notes_arg, NOTES_EQ_ARG);
 		res = parse_string(opt_notes_arg + STRING_SIZE(NOTES_EQ_ARG), arg,
 				   sizeof(opt_notes_arg) - STRING_SIZE(NOTES_EQ_ARG));
@@ -494,12 +589,12 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 		const char *type = option->type + STRING_SIZE("enum ");
 		const struct enum_map *map = find_enum_map(type);
 
-		return parse_enum(option->value, arg, map);
+		return parse_enum(name, option->value, arg, map);
 	}
 
 	if (!strcmp(option->type, "int")) {
 		if (strstr(name, "title-overflow")) {
-			bool enabled = FALSE;
+			bool enabled = false;
 			int *value = option->value;
 
 			/* We try to parse it as a boolean (and set the
@@ -523,40 +618,53 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 			return parse_int(option->value, arg, 0, 1024);
 	}
 
+	if (!strcmp(option->type, "const char *")) {
+		const char *alloc = NULL;
+		const char **value = option->value;
+
+		if (strlen(arg)) {
+			if (arg[0] == '"' && arg[strlen(arg) - 1] == '"')
+				alloc = strndup(arg + 1, strlen(arg + 1) - 1);
+			else
+				alloc = strdup(arg);
+			if (!alloc)
+				return ERROR_OUT_OF_MEMORY;
+		}
+
+		free((void *) *value);
+		*value = alloc;
+		return SUCCESS;
+	}
+
 	return error("Unhandled option: %s", name);
 }
 
-struct view_config {
-	const char *name;
-	const char ***argv;
-};
-
-static struct view_config view_configs[] = {
-	{ "blame-view", &opt_blame_view },
-	{ "blob-view", &opt_blob_view },
-	{ "diff-view", &opt_diff_view },
-	{ "grep-view", &opt_grep_view },
-	{ "log-view", &opt_log_view },
-	{ "main-view", &opt_main_view },
-	{ "pager-view", &opt_pager_view },
-	{ "refs-view", &opt_refs_view },
-	{ "stage-view", &opt_stage_view },
-	{ "stash-view", &opt_stash_view },
-	{ "status-view", &opt_status_view },
-	{ "tree-view", &opt_tree_view },
-};
-
 static enum status_code
-check_view_config(struct option_info *option, const char *argv[])
+parse_view_settings(struct view_column **view_column, const char *name_, const char *argv[])
 {
-	const char *name = enum_name(option->name);
-	int i;
+	char buf[SIZEOF_STR];
+	const char *name = enum_name_copy(buf, sizeof(buf), name_) ? buf : name_;
+	const char *prefixed;
 
-	for (i = 0; i < ARRAY_SIZE(view_configs); i++)
-		if (!strcmp(name, view_configs[i].name))
-			return parse_view_config(name, argv);
+	if ((prefixed = strstr(name, "-view-"))) {
+		const char *column_name = prefixed + STRING_SIZE("-view-");
+		size_t column_namelen = strlen(column_name);
+		enum view_column_type type;
 
-	return SUCCESS;
+		for (type = 0; type < view_column_type_map->size; type++) {
+			const struct enum_map_entry *column = &view_column_type_map->entries[type];
+
+			if (enum_equals(*column, column_name, column_namelen))
+				return parse_view_column_config(name, type, NULL, argv);
+
+			if (enum_equals_prefix(*column, column_name, column_namelen))
+				return parse_view_column_config(name, type,
+								column_name + column->namelen + 1,
+								argv);
+		}
+	}
+
+	return parse_view_config(view_column, name, argv);
 }
 
 /* Wants: name = value */
@@ -564,29 +672,30 @@ static enum status_code
 option_set_command(int argc, const char *argv[])
 {
 	struct option_info *option;
+	enum status_code code;
 
-	if (argc < 3)
+	if (argc < 2)
 		return error("Invalid set command: set option = value");
 
 	if (strcmp(argv[1], "="))
 		return error("No value assigned to %s", argv[0]);
 
-	if (!strcmp(argv[0], "reference-format"))
-		return parse_ref_formats(argv + 2);
-
-	option = find_option_info(option_info, ARRAY_SIZE(option_info), argv[0]);
+	option = find_option_info(option_info, ARRAY_SIZE(option_info), "", argv[0]);
 	if (option) {
-		enum status_code code;
-
 		if (option->seen)
 			return SUCCESS;
 
-		if (!strcmp(option->type, "const char **")) {
-			code = check_view_config(option, argv + 2);
-			if (code != SUCCESS)
-				return code;
+		if (!strcmp(option->type, "const char **"))
 			return parse_args(option->value, argv + 2);
-		}
+
+		if (argc < 3)
+			return error("Invalid set command: set option = value");
+
+		if (!strcmp(option->type, "view_settings"))
+			return parse_view_settings(option->value, argv[0], argv + 2);
+
+		if (!strcmp(option->type, "struct ref_format **"))
+			return parse_ref_formats(option->value, argv + 2);
 
 		code = parse_option(option, "", argv[2]);
 		if (code == SUCCESS && argc != 3)
@@ -619,6 +728,9 @@ option_set_command(int argc, const char *argv[])
 
 		if (!strcmp(argv[0], "read-git-colors"))
 			return error("read-git-colors has been obsoleted by the git-colors option");
+
+		if (!strcmp(argv[0], "cmdline-args"))
+			return error("cmdline-args is obsolete; use view-specific options instead, e.g. main-options");
 	}
 
 	return error("Unknown option name: %s", argv[0]);
@@ -628,7 +740,7 @@ option_set_command(int argc, const char *argv[])
 static enum status_code
 option_bind_command(int argc, const char *argv[])
 {
-	struct key key[1];
+	struct key key[16];
 	size_t keys = 0;
 	enum request request;
 	struct keymap *keymap;
@@ -663,10 +775,12 @@ option_bind_command(int argc, const char *argv[])
 		static const char *toggles[][2] = {
 			{ "diff-context-down",		"diff-context" },
 			{ "diff-context-up",		"diff-context" },
+			{ "stage-next",			":/^@@" },
 			{ "toggle-author",		"author" },
 			{ "toggle-changes",		"show-changes" },
 			{ "toggle-commit-order",	"show-commit-order" },
 			{ "toggle-date",		"date" },
+			{ "toggle-files",		"file-filter" },
 			{ "toggle-file-filter",		"file-filter" },
 			{ "toggle-file-size",		"file-size" },
 			{ "toggle-filename",		"filename" },
@@ -676,6 +790,7 @@ option_bind_command(int argc, const char *argv[])
 			{ "toggle-lineno",		"line-number" },
 			{ "toggle-refs",		"commit-title-refs" },
 			{ "toggle-rev-graph",		"commit-title-graph" },
+			{ "toggle-show-changes",	"show-changes" },
 			{ "toggle-sort-field",		"sort-field" },
 			{ "toggle-sort-order",		"sort-order" },
 			{ "toggle-title-overflow",	"commit-title-overflow" },
@@ -698,12 +813,16 @@ option_bind_command(int argc, const char *argv[])
 			const char *action = toggles[alias][0];
 			const char *arg = prefixcmp(action, "diff-context-")
 					? NULL : (strstr(action, "-down") ? "-1" : "+1");
-			const char *toggle[] = { ":toggle", toggles[alias][1], arg, NULL};
-			enum status_code code = add_run_request(keymap, key, keys, toggle);
+			const char *mapped = toggles[alias][1];
+			const char *toggle[] = { ":toggle", mapped, arg, NULL};
+			const char *other[] = { mapped, NULL };
+			const char **prompt = *mapped == ':' ? other : toggle;
+			enum status_code code = add_run_request(keymap, key, keys, prompt);
 
 			if (code == SUCCESS)
-				code = error("%s has been replaced by `:toggle %s%s%s'",
-					     action, toggles[alias][1],
+				code = error("%s has been replaced by `%s%s%s%s'",
+					     action, prompt == other ? mapped : ":toggle ",
+					     prompt == other ? "" : mapped,
 					     arg ? " " : "", arg ? arg : "");
 			return code;
 		}
@@ -756,7 +875,7 @@ struct config_state {
 	bool errors;
 };
 
-static int
+static enum status_code
 read_option(char *opt, size_t optlen, char *value, size_t valuelen, void *data)
 {
 	struct config_state *config = data;
@@ -766,7 +885,7 @@ read_option(char *opt, size_t optlen, char *value, size_t valuelen, void *data)
 	 * only ensure opt and value are split at first " \t". */
 	optlen = strcspn(opt, "#");
 	if (optlen == 0)
-		return OK;
+		return SUCCESS;
 
 	if (opt[optlen] == 0) {
 		/* Look for comment endings in the value. */
@@ -788,17 +907,17 @@ read_option(char *opt, size_t optlen, char *value, size_t valuelen, void *data)
 	if (status != SUCCESS) {
 		warn("%s:%zu: %s", config->path, config->lineno,
 		     get_status_message(status));
-		config->errors = TRUE;
+		config->errors = true;
 	}
 
 	/* Always keep going if errors are encountered. */
-	return OK;
+	return SUCCESS;
 }
 
 static enum status_code
 load_option_file(const char *path)
 {
-	struct config_state config = { path, 0, FALSE };
+	struct config_state config = { path, 0, false };
 	struct io io;
 	char buf[SIZEOF_STR];
 
@@ -820,18 +939,18 @@ load_option_file(const char *path)
 		 * system tigrc is detected properly. */
 		if (io_error(&io) == ENOENT)
 			return ERROR_FILE_DOES_NOT_EXIST;
-		return error("Error loading file %s: %s", path, strerror(io_error(&io)));
+		return error("Error loading file %s: %s", path, io_strerror(&io));
 	}
 
-	if (io_load_span(&io, " \t", &config.lineno, read_option, &config) == ERR ||
-	    config.errors == TRUE)
+	if (io_load_span(&io, " \t", &config.lineno, read_option, &config) != SUCCESS ||
+	    config.errors == true)
 		warn("Errors while loading %s.", path);
 	return SUCCESS;
 }
 
 extern const char *builtin_config;
 
-int
+enum status_code
 load_options(void)
 {
 	const char *tigrc_user = getenv("TIGRC_USER");
@@ -839,8 +958,9 @@ load_options(void)
 	const char *tig_diff_opts = getenv("TIG_DIFF_OPTS");
 	const bool diff_opts_from_args = !!opt_diff_options;
 	bool custom_tigrc_system = !!tigrc_system;
+	char buf[SIZEOF_STR];
 
-	opt_file_filter = TRUE;
+	opt_file_filter = true;
 	if (!find_option_info_by_value(&opt_diff_context)->seen)
 		opt_diff_context = -3;
 
@@ -849,18 +969,30 @@ load_options(void)
 
 	if (!*tigrc_system ||
 	    (load_option_file(tigrc_system) == ERROR_FILE_DOES_NOT_EXIST && !custom_tigrc_system)) {
-		struct config_state config = { "<built-in>", 0, FALSE };
+		struct config_state config = { "<built-in>", 0, false };
 		struct io io;
 
 		if (!io_from_string(&io, builtin_config))
-			die("Failed to get built-in config");
-		if (!io_load_span(&io, " \t", &config.lineno, read_option, &config) == ERR || config.errors == TRUE)
-			die("Error in built-in config");
+			return error("Failed to get built-in config");
+		if (io_load_span(&io, " \t", &config.lineno, read_option, &config) != SUCCESS || config.errors == true)
+			return error("Error in built-in config");
 	}
 
-	if (!tigrc_user)
-		tigrc_user = "~/.tigrc";
-	load_option_file(tigrc_user);
+	if (tigrc_user) {
+		load_option_file(tigrc_user);
+	} else {
+		const char *xdg_config_home = getenv("XDG_CONFIG_HOME");
+
+		if (!xdg_config_home || !*xdg_config_home)
+			tigrc_user = "~/.config/tig/config";
+		else if (!string_format(buf, "%s/tig/config", xdg_config_home))
+			return error("Failed to expand $XDG_CONFIG_HOME");
+		else
+			tigrc_user = buf;
+
+		if (load_option_file(tigrc_user) == ERROR_FILE_DOES_NOT_EXIST)
+			load_option_file(TIG_USER_CONFIG);
+	}
 
 	if (!diff_opts_from_args && tig_diff_opts && *tig_diff_opts) {
 		static const char *diff_opts[SIZEOF_ARG] = { NULL };
@@ -869,12 +1001,244 @@ load_options(void)
 
 		if (!string_format(buf, "%s", tig_diff_opts) ||
 		    !argv_from_string(diff_opts, &argc, buf))
-			die("TIG_DIFF_OPTS contains too many arguments");
+			return error("TIG_DIFF_OPTS contains too many arguments");
 		else if (!argv_copy(&opt_diff_options, diff_opts))
-			die("Failed to format TIG_DIFF_OPTS arguments");
+			return error("Failed to format TIG_DIFF_OPTS arguments");
 	}
 
-	return OK;
+	return SUCCESS;
+}
+
+const char *
+format_option_value(const struct option_info *option, char buf[], size_t bufsize)
+{
+	buf[0] = 0;
+
+	if (!strcmp(option->type, "bool")) {
+		bool *opt = option->value;
+
+		if (string_nformat(buf, bufsize, NULL, "%s", *opt ? "yes" : "no"))
+			return buf;
+
+	} else if (!strncmp(option->type, "enum", 4)) {
+		const char *type = option->type + STRING_SIZE("enum ");
+		enum author *opt = option->value;
+		const struct enum_map *map = find_enum_map(type);
+
+		if (enum_name_copy(buf, bufsize, map->entries[*opt].name))
+			return buf;
+
+	} else if (!strcmp(option->type, "int")) {
+		int *opt = option->value;
+
+		if (opt == &opt_diff_context && *opt < 0)
+			*opt = -*opt;
+
+		if (string_nformat(buf, bufsize, NULL, "%d", *opt))
+			return buf;
+
+	} else if (!strcmp(option->type, "double")) {
+		double *opt = option->value;
+
+		if (*opt >= 1) {
+			if (string_nformat(buf, bufsize, NULL, "%d", (int) *opt))
+				return buf;
+
+		} else if (string_nformat(buf, bufsize, NULL, "%.0f%%", (*opt) * 100)) {
+			return buf;
+		}
+
+	} else if (!strcmp(option->type, "const char *")) {
+		const char **opt = option->value;
+		size_t bufpos = 0;
+
+		if (!*opt)
+			return "\"\"";
+		if (!string_nformat(buf, bufsize, &bufpos, "\"%s\"", *opt))
+			return NULL;
+		return buf;
+
+	} else if (!strcmp(option->type, "const char **")) {
+		const char *sep = "";
+		const char ***opt = option->value;
+		size_t bufpos = 0;
+		int i;
+
+		for (i = 0; (*opt) && (*opt)[i]; i++) {
+			const char *arg = (*opt)[i];
+
+			if (!string_nformat(buf, bufsize, &bufpos, "%s%s", sep, arg))
+				return NULL;
+
+			sep = " ";
+		}
+
+		return buf;
+
+	} else if (!strcmp(option->type, "struct ref_format **")) {
+		struct ref_format ***opt = option->value;
+
+		if (format_ref_formats(*opt, buf, bufsize) == SUCCESS)
+			return buf;
+
+	} else if (!strcmp(option->type, "view_settings")) {
+		struct view_column **opt = option->value;
+
+		if (format_view_config(*opt, buf, bufsize) == SUCCESS)
+			return buf;
+
+	} else {
+		if (string_nformat(buf, bufsize, NULL, "<%s>", option->type))
+			return buf;
+	}
+
+	return NULL;
+}
+
+static bool
+save_option_settings(FILE *file)
+{
+	char buf[SIZEOF_STR];
+	int i;
+
+	if (!io_fprintf(file, "%s", "\n## Settings\n"))
+		return false;
+
+	for (i = 0; i < ARRAY_SIZE(option_info); i++) {
+		struct option_info *option = &option_info[i];
+		const char *name = enum_name(option->name);
+		const char *value = format_option_value(option, buf, sizeof(buf));
+
+		if (!value)
+			return false;
+
+		if (!suffixcmp(name, strlen(name), "-args"))
+			continue;
+
+		if (!io_fprintf(file, "\nset %-25s = %s", name, value))
+			return false;
+	}
+
+	return true;
+}
+
+static bool
+save_option_keybinding(void *data, const char *group, struct keymap *keymap,
+		       enum request request, const char *key,
+		       const struct request_info *req_info,
+		       const struct run_request *run_req)
+{
+	FILE *file = data;
+
+	if (group && !io_fprintf(file, "\n# %s", group))
+		return false;
+
+	if (!io_fprintf(file, "\nbind %-10s %-15s ", enum_name(keymap->name), key))
+		return false;
+
+	if (req_info) {
+		return io_fprintf(file, "%s", enum_name(req_info->name));
+
+	} else {
+		const char *sep = format_run_request_flags(run_req);
+		int i;
+
+		for (i = 0; run_req->argv[i]; i++) {
+			if (!io_fprintf(file, "%s%s", sep, run_req->argv[i]))
+				return false;
+			sep = " ";
+		}
+
+		return true;
+	}
+}
+
+static bool
+save_option_keybindings(FILE *file)
+{
+	if (!io_fprintf(file, "%s", "\n\n## Keybindings\n"))
+		return false;
+
+	return foreach_key(save_option_keybinding, file, false);
+}
+
+static bool
+save_option_color_name(FILE *file, int color)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(color_map); i++)
+		if (color_map[i].value == color)
+			return io_fprintf(file, " %-8s", enum_name(color_map[i].name));
+
+	return io_fprintf(file, " color%d", color);
+}
+
+static bool
+save_option_color_attr(FILE *file, int attr)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(attr_map); i++)
+		if ((attr & attr_map[i].value) &&
+		    !io_fprintf(file, " %s", enum_name(attr_map[i].name)))
+			return false;
+
+	return true;
+}
+
+static bool
+save_option_color(void *data, const struct line_rule *rule)
+{
+	FILE *file = data;
+	const struct line_info *info;
+
+	for (info = &rule->info; info; info = info->next) {
+		const char *prefix = info->prefix ? info->prefix : "";
+		const char *prefix_sep = info->prefix ? "." : "";
+		const char *quote = *rule->line ? "\"" : "";
+		const char *name = *rule->line ? rule->line : enum_name(rule->name);
+		int name_width = strlen(prefix) + strlen(prefix_sep) + 2 * strlen(quote) + strlen(name);
+		int padding = name_width > 30 ? 0 : 30 - name_width;
+
+		if (!io_fprintf(file, "\ncolor %s%s%s%s%s%-*s",
+				      prefix, prefix_sep, quote, name, quote, padding, "")
+		    || !save_option_color_name(file, info->fg)
+		    || !save_option_color_name(file, info->bg)
+		    || !save_option_color_attr(file, info->attr))
+			return false;
+	}
+
+	return true;
+}
+
+static bool
+save_option_colors(FILE *file)
+{
+	if (!io_fprintf(file, "%s", "\n\n## Colors\n"))
+		return false;
+
+	return foreach_line_rule(save_option_color, file);
+}
+
+enum status_code
+save_options(const char *path)
+{
+	int fd = open(path, O_WRONLY | O_CREAT | O_EXCL, 0666);
+	FILE *file = fd != -1 ? fdopen(fd, "w") : NULL;
+	enum status_code code = SUCCESS;
+
+	if (!file)
+		return error("%s", strerror(errno));
+
+	if (!io_fprintf(file, "%s", "# Saved by Tig\n")
+	    || !save_option_settings(file)
+	    || !save_option_keybindings(file)
+	    || !save_option_colors(file))
+		code = error("Write returned an error");
+
+	fclose(file);
+	return code;
 }
 
 /*
@@ -931,11 +1295,11 @@ set_work_tree(const char *value)
 		die("Failed to chdir(%s): %s", value, strerror(errno));
 	if (!getcwd(cwd, sizeof(cwd)))
 		die("Failed to get cwd path: %s", strerror(errno));
-	if (setenv("GIT_WORK_TREE", cwd, TRUE))
+	if (setenv("GIT_WORK_TREE", cwd, true))
 		die("Failed to set GIT_WORK_TREE to '%s'", cwd);
-	if (setenv("GIT_DIR", repo.git_dir, TRUE))
+	if (setenv("GIT_DIR", repo.git_dir, true))
 		die("Failed to set GIT_DIR to '%s'", repo.git_dir);
-	repo.is_inside_work_tree = TRUE;
+	repo.is_inside_work_tree = true;
 }
 
 static struct line_info *
@@ -943,7 +1307,7 @@ parse_git_color_option(struct line_info *info, char *value)
 {
 	const char *argv[SIZEOF_ARG];
 	int argc = 0;
-	bool first_color = TRUE;
+	bool first_color = true;
 	int i;
 
 	if (!argv_from_string(argv, &argc, value))
@@ -964,7 +1328,7 @@ parse_git_color_option(struct line_info *info, char *value)
 				info->fg = attr;
 			else
 				info->bg = attr;
-			first_color = FALSE;
+			first_color = false;
 		}
 	}
 	return info;
@@ -973,7 +1337,7 @@ parse_git_color_option(struct line_info *info, char *value)
 static void
 set_git_color_option(const char *name, char *value)
 {
-	struct line_info parsed = {};
+	struct line_info parsed = {0};
 	struct line_info *color = NULL;
 	size_t namelen = strlen(name);
 	int i;
@@ -982,7 +1346,7 @@ set_git_color_option(const char *name, char *value)
 		return;
 
 	for (i = 0; opt_git_colors[i]; i++) {
-		struct line_rule rule = {};
+		struct line_rule rule = {0};
 		const char *prefix = NULL;
 		struct line_info *info;
 		const char *alias = opt_git_colors[i];
@@ -1016,14 +1380,14 @@ set_encoding(struct encoding **encoding_ref, const char *arg, bool priority)
 		encoding_arg[0] = 0;
 }
 
-static int
+static enum status_code
 read_repo_config_option(char *name, size_t namelen, char *value, size_t valuelen, void *data)
 {
 	if (!strcmp(name, "i18n.commitencoding"))
-		set_encoding(&default_encoding, value, FALSE);
+		set_encoding(&default_encoding, value, false);
 
 	else if (!strcmp(name, "gui.encoding"))
-		set_encoding(&default_encoding, value, TRUE);
+		set_encoding(&default_encoding, value, true);
 
 	else if (!strcmp(name, "core.editor"))
 		string_ncopy(opt_editor, value, valuelen);
@@ -1034,6 +1398,9 @@ read_repo_config_option(char *name, size_t namelen, char *value, size_t valuelen
 	else if (!strcmp(name, "core.abbrev"))
 		parse_int(&opt_id_width, value, 0, SIZEOF_REV - 1);
 
+	else if (!strcmp(name, "diff.noprefix"))
+		parse_bool(&opt_diff_noprefix, value);
+
 	else if (!prefixcmp(name, "tig.color."))
 		set_repo_config_option(name + 10, value, option_color_command);
 
@@ -1053,12 +1420,16 @@ read_repo_config_option(char *name, size_t namelen, char *value, size_t valuelen
 	else if (!strcmp(name, "diff.context")) {
 		if (!find_option_info_by_value(&opt_diff_context)->seen)
 			opt_diff_context = -atoi(value);
+
+	} else if (!strcmp(name, "format.pretty")) {
+		if (!prefixcmp(value, "format:") && strstr(value, "%C("))
+			argv_append(&opt_log_options, "--pretty=medium");
 	}
 
-	return OK;
+	return SUCCESS;
 }
 
-int
+enum status_code
 load_git_config(void)
 {
 	const char *config_list_argv[] = { "git", "config", "--list", NULL };
