@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -12,6 +12,7 @@
  */
 
 #include "tig/tig.h"
+#include "tig/repo.h"
 #include "tig/view.h"
 #include "tig/draw.h"
 #include "tig/display.h"
@@ -33,14 +34,19 @@ prompt_input(const char *prompt, struct input *input)
 	struct key key;
 	size_t promptlen = strlen(prompt);
 	int pos = 0, chars = 0;
+	int last_buf_length = promptlen ? -1 : promptlen;
 
-	input->buf[pos] = 0;
+	input->buf[0] = 0;
 
 	while (status == INPUT_OK || status == INPUT_SKIP) {
+		int buf_length = strlen(input->buf) + promptlen;
+		int offset = pos || buf_length != last_buf_length ? pos + promptlen : -1;
 
-		update_status("%s%.*s", prompt, pos, input->buf);
+		last_buf_length = buf_length;
+		if (offset >= 0)
+			update_status("%s%.*s", prompt, pos, input->buf);
 
-		if (get_input(pos + promptlen, &key, FALSE) == OK) {
+		if (get_input(offset, &key) == OK) {
 			int len = strlen(key.data.bytes);
 
 			if (pos + len >= sizeof(input->buf)) {
@@ -92,17 +98,14 @@ prompt_input(const char *prompt, struct input *input)
 	return input->buf;
 }
 
-static enum input_status
+enum input_status
 prompt_default_handler(struct input *input, struct key *key)
 {
-	if (key->modifiers.multibytes)
-		return INPUT_SKIP;
-
-	switch (key->data.value) {
+	switch (key_to_value(key)) {
 	case KEY_RETURN:
 	case KEY_ENTER:
 	case '\n':
-		return *input->buf ? INPUT_STOP : INPUT_CANCEL;
+		return *input->buf || input->allow_empty ? INPUT_STOP : INPUT_CANCEL;
 
 	case KEY_BACKSPACE:
 		return *input->buf ? INPUT_DELETE : INPUT_CANCEL;
@@ -131,10 +134,10 @@ bool
 prompt_yesno(const char *prompt)
 {
 	char prompt2[SIZEOF_STR];
-	struct input input = { prompt_yesno_handler, NULL };
+	struct input input = { prompt_yesno_handler, false, NULL };
 
 	if (!string_format(prompt2, "%s [Yy/Nn]", prompt))
-		return FALSE;
+		return false;
 
 	return !!prompt_input(prompt2, &input);
 }
@@ -163,10 +166,11 @@ read_prompt_handler(struct input *input, struct key *key)
 }
 
 char *
-read_prompt_incremental(const char *prompt, bool edit_mode, input_handler handler, void *data)
+read_prompt_incremental(const char *prompt, bool edit_mode, bool allow_empty, input_handler handler, void *data)
 {
 	static struct incremental_input incremental = { { read_prompt_handler } };
 
+	incremental.input.allow_empty = allow_empty;
 	incremental.input.data = data;
 	incremental.handler = handler;
 	incremental.edit_mode = edit_mode;
@@ -178,10 +182,7 @@ read_prompt_incremental(const char *prompt, bool edit_mode, input_handler handle
 static void
 readline_display(void)
 {
-	wmove(status_win, 0, 0);
-	waddstr(status_win, rl_display_prompt);
-	waddstr(status_win, rl_line_buffer);
-	wclrtoeol(status_win);
+	update_status("%s%s", rl_display_prompt, rl_line_buffer);
 	wmove(status_win, 0, strlen(rl_display_prompt) + rl_point);
 	wrefresh(status_win);
 }
@@ -190,8 +191,8 @@ static char *
 readline_variable_generator(const char *text, int state)
 {
 	static const char *vars[] = {
-#define FORMAT_VAR(name, ifempty, initval) "%(" #name ")"
-		ARGV_ENV_INFO(FORMAT_VAR),
+#define FORMAT_VAR(type, name, ifempty, initval) "%(" #name ")",
+		ARGV_ENV_INFO(FORMAT_VAR)
 #undef FORMAT_VAR
 		NULL
 	};
@@ -233,6 +234,10 @@ readline_action_generator(const char *text, int state)
 		"bind",
 		"set",
 		"toggle",
+		"goto",
+		"save-display",
+		"save-options",
+		"exec",
 #define REQ_GROUP(help)
 #define REQ_(req, help)	#req
 		REQ_INFO,
@@ -359,7 +364,7 @@ readline_toggle_generator(const char *text, int state)
 static int
 readline_getc(FILE *stream)
 {
-	return getc(opt_tty);
+	return get_input_char();
 }
 
 static char **
@@ -449,7 +454,13 @@ read_prompt(const char *prompt)
 	}
 
 	line = readline(prompt);
-	if (line && *line)
+
+	if (line && !*line) {
+		free(line);
+		line = NULL;
+	}
+
+	if (line)
 		add_history(line);
 
 	return line;
@@ -464,7 +475,7 @@ prompt_init(void)
 char *
 read_prompt(const char *prompt)
 {
-	return read_prompt_incremental(prompt, TRUE, NULL, NULL);
+	return read_prompt_incremental(prompt, true, false, NULL, NULL);
 }
 
 void
@@ -493,7 +504,7 @@ prompt_menu(const char *prompt, const struct menu_item *items, int *selected)
 		update_status("%s (%d of %d) %s%s", prompt, *selected + 1, size,
 			      item->hotkey ? hotkey : "", item->text);
 
-		switch (get_input(COLS - 1, &key, FALSE)) {
+		switch (get_input(COLS - 1, &key)) {
 		case KEY_RETURN:
 		case KEY_ENTER:
 		case '\n':
@@ -531,11 +542,9 @@ prompt_menu(const char *prompt, const struct menu_item *items, int *selected)
 	return status != INPUT_CANCEL;
 }
 
-struct prompt_toggle {
-	const char *name;
-	const char *type;
-	enum view_flag flags;
-	void *opt;
+static struct option_info option_toggles[] = {
+#define DEFINE_OPTION_TOGGLES(name, type, flags) { #name, STRING_SIZE(#name), #type, &opt_ ## name, flags },
+	OPTION_INFO(DEFINE_OPTION_TOGGLES)
 };
 
 static bool
@@ -543,15 +552,15 @@ find_arg(const char *argv[], const char *arg)
 {
 	int i;
 
-	for (i = 0; argv[i]; i++)
+	for (i = 0; argv && argv[i]; i++)
 		if (!strcmp(argv[i], arg))
-			return TRUE;
-	return FALSE;
+			return true;
+	return false;
 }
 
 static enum status_code
 prompt_toggle_option(struct view *view, const char *argv[], const char *prefix,
-		     struct prompt_toggle *toggle, enum view_flag *flags)
+		     struct option_info *toggle, enum view_flag *flags)
 {
 	char name[SIZEOF_STR];
 
@@ -561,7 +570,7 @@ prompt_toggle_option(struct view *view, const char *argv[], const char *prefix,
 	*flags = toggle->flags;
 
 	if (!strcmp(toggle->type, "bool")) {
-		bool *opt = toggle->opt;
+		bool *opt = toggle->value;
 
 		*opt = !*opt;
 		if (opt == &opt_mouse)
@@ -570,7 +579,7 @@ prompt_toggle_option(struct view *view, const char *argv[], const char *prefix,
 
 	} else if (!strncmp(toggle->type, "enum", 4)) {
 		const char *type = toggle->type + STRING_SIZE("enum ");
-		enum author *opt = toggle->opt;
+		enum author *opt = toggle->value;
 		const struct enum_map *map = find_enum_map(type);
 
 		*opt = (*opt + 1) % map->size;
@@ -579,7 +588,7 @@ prompt_toggle_option(struct view *view, const char *argv[], const char *prefix,
 	} else if (!strcmp(toggle->type, "int")) {
 		const char *arg = argv[2] ? argv[2] : "1";
 		int diff = atoi(arg);
-		int *opt = toggle->opt;
+		int *opt = toggle->value;
 
 		if (!diff)
 			diff = *arg == '-' ? -1 : 1;
@@ -605,7 +614,7 @@ prompt_toggle_option(struct view *view, const char *argv[], const char *prefix,
 
 	} else if (!strcmp(toggle->type, "double")) {
 		const char *arg = argv[2] ? argv[2] : "1.0";
-		double *opt = toggle->opt;
+		double *opt = toggle->value;
 		int sign = 1;
 		double diff;
 
@@ -621,13 +630,19 @@ prompt_toggle_option(struct view *view, const char *argv[], const char *prefix,
 		return success("set %s = %.2f", name, *opt);
 
 	} else if (!strcmp(toggle->type, "const char **")) {
-		const char ***opt = toggle->opt;
-		bool found = TRUE;
+		const char ***opt = toggle->value;
+		bool found = true;
 		int i;
 
+		if (argv_size(argv) <= 2) {
+			argv_free(*opt);
+			(*opt)[0] = NULL;
+			return SUCCESS;
+		}
+
 		for (i = 2; argv[i]; i++) {
 			if (!find_arg(*opt, argv[i])) {
-				found = FALSE;
+				found = false;
 				break;
 			}
 		}
@@ -657,46 +672,15 @@ prompt_toggle_option(struct view *view, const char *argv[], const char *prefix,
 	}
 }
 
-static struct prompt_toggle *
-find_prompt_toggle(struct prompt_toggle toggles[], size_t toggles_size,
-		   const char *prefix, const char *name, size_t namelen)
-{
-	char prefixed[SIZEOF_STR];
-	int i;
-
-	if (*prefix && namelen == strlen(prefix) &&
-	    !string_enum_compare(prefix, name, namelen)) {
-		name = "display";
-		namelen = strlen(name);
-	}
-
-	for (i = 0; i < toggles_size; i++) {
-		struct prompt_toggle *toggle = &toggles[i];
-
-		if (namelen == strlen(toggle->name) &&
-		    !string_enum_compare(toggle->name, name, namelen))
-			return toggle;
-
-		if (enum_name_prefixed(prefixed, sizeof(prefixed), prefix, toggle->name) &&
-		    namelen == strlen(prefixed) &&
-		    !string_enum_compare(prefixed, name, namelen))
-			return toggle;
-	}
-
-	return NULL;
-}
-
 static enum status_code
 prompt_toggle(struct view *view, const char *argv[], enum view_flag *flags)
 {
-	struct prompt_toggle option_toggles[] = {
-#define DEFINE_OPTION_TOGGLES(name, type, flags) { #name, #type, flags, &opt_ ## name },
-		OPTION_INFO(DEFINE_OPTION_TOGGLES)
-	};
 	const char *option = argv[1];
 	size_t optionlen = option ? strlen(option) : 0;
-	struct prompt_toggle *toggle;
+	struct option_info template;
+	struct option_info *toggle;
 	struct view_column *column;
+	const char *column_name;
 
 	if (!option)
 		return error("%s", "No option name given to :toggle");
@@ -716,32 +700,38 @@ prompt_toggle(struct view *view, const char *argv[], enum view_flag *flags)
 		}
 	}
 
-	toggle = find_prompt_toggle(option_toggles, ARRAY_SIZE(option_toggles),
-				    "", option, optionlen);
+	toggle = find_option_info(option_toggles, ARRAY_SIZE(option_toggles), "", option);
 	if (toggle)
 		return prompt_toggle_option(view, argv, "", toggle, flags);
 
-#define DEFINE_COLUMN_OPTIONS_TOGGLE(name, type, flags) \
-	{ #name, #type, flags, &opt->name },
-
-#define DEFINE_COLUMN_OPTIONS_CHECK(name, id, options) \
-	if (column->type == VIEW_COLUMN_##id) { \
-		struct name##_options *opt = &column->opt.name; \
-		struct prompt_toggle toggles[] = { \
-			options(DEFINE_COLUMN_OPTIONS_TOGGLE) \
-		}; \
-		toggle = find_prompt_toggle(toggles, ARRAY_SIZE(toggles), #name, option, optionlen); \
-		if (toggle) \
-			return prompt_toggle_option(view, argv, #name, toggle, flags); \
-	}
-
 	for (column = view->columns; column; column = column->next) {
-		COLUMN_OPTIONS(DEFINE_COLUMN_OPTIONS_CHECK);
+		toggle = find_column_option_info(column->type, &column->opt, option, &template, &column_name);
+		if (toggle)
+			return prompt_toggle_option(view, argv, column_name, toggle, flags);
 	}
 
 	return error("`:toggle %s` not supported", option);
 }
 
+static void
+prompt_update_display(enum view_flag flags)
+{
+	struct view *view;
+	int i;
+
+	if (flags & VIEW_RESET_DISPLAY) {
+		resize_display();
+		redraw_display(true);
+	}
+
+	foreach_displayed_view(view, i) {
+		if (view_has_flags(view, flags) && view_can_refresh(view))
+			reload_view(view);
+		else
+			redraw_view(view);
+	}
+}
+
 enum request
 run_prompt_command(struct view *view, const char *argv[])
 {
@@ -764,28 +754,7 @@ run_prompt_command(struct view *view, const char *argv[])
 			report("Unable to parse '%s' as a line number", cmd);
 		}
 	} else if (iscommit(cmd)) {
-		int lineno;
-
-		if (!(view->ops->column_bits & view_column_bit(ID))) {
-			report("Jumping to commits is not supported by the %s view", view->name);
-			return REQ_NONE;
-		}
-
-		for (lineno = 0; lineno < view->lines; lineno++) {
-			struct view_column_data column_data = {};
-			struct line *line = &view->line[lineno];
-
-			if (view->ops->get_column_data(view, line, &column_data) &&
-			    column_data.id &&
-			    !strncasecmp(column_data.id, cmd, cmdlen)) {
-				string_ncopy(view->env->search, cmd, cmdlen);
-				select_view_line(view, lineno);
-				report_clear();
-				return REQ_NONE;
-			}
-		}
-
-		report("Unable to find commit '%s'", view->env->search);
+		goto_id(view, cmd, true, true);
 		return REQ_NONE;
 
 	} else if (cmdlen > 1 && (cmd[0] == '/' || cmd[0] == '?')) {
@@ -800,7 +769,7 @@ run_prompt_command(struct view *view, const char *argv[])
 			return cmd[0] == '/' ? REQ_FIND_NEXT : REQ_FIND_PREV;
 
 		string_ncopy(view->env->search, search + 1, strlen(search + 1));
-		return cmd[0] == '/' ? REQ_SEARCH : REQ_SEARCH_BACK;
+		return cmd[0] == '/' ? REQ_FIND_NEXT : REQ_FIND_PREV;
 
 	} else if (cmdlen > 1 && cmd[0] == '!') {
 		struct view *next = &pager_view;
@@ -808,7 +777,7 @@ run_prompt_command(struct view *view, const char *argv[])
 
 		/* Trim the leading '!'. */
 		argv[0] = cmd + 1;
-		copied = argv_format(view->env, &next->argv, argv, FALSE, TRUE);
+		copied = argv_format(view->env, &next->argv, argv, false, true);
 		argv[0] = cmd;
 
 		if (!copied) {
@@ -823,40 +792,78 @@ run_prompt_command(struct view *view, const char *argv[])
 			open_pager_view(view, OPEN_PREPARED | OPEN_WITH_STDERR);
 		}
 
+	} else if (!strcmp(cmd, "goto")) {
+		if (!argv[1] || !strlen(argv[1]))
+			report("goto requires an argument");
+		else
+			goto_id(view, argv[1], true, true);
+		return REQ_NONE;
+
+	} else if (!strcmp(cmd, "save-display")) {
+		const char *path = argv[1] ? argv[1] : "tig-display.txt";
+
+		if (!save_display(path))
+			report("Failed to save screen to %s", path);
+		else
+			report("Saved screen to %s", path);
+
+	} else if (!strcmp(cmd, "save-options")) {
+		const char *path = argv[1] ? argv[1] : "tig-options.txt";
+		enum status_code code = save_options(path);
+
+		if (code != SUCCESS)
+			report("Failed to save options: %s", get_status_message(code));
+		else
+			report("Saved options to %s", path);
+
+	} else if (!strcmp(cmd, "exec")) {
+		// argv may be allocated and mutations below will cause
+		// free() to error out so backup and restore. :(
+		const char *cmd = argv[1];
+		struct run_request req = { view->keymap, {0}, argv + 1 };
+		enum status_code code = parse_run_request_flags(&req.flags, argv + 1);
+
+		if (code != SUCCESS) {
+			argv[1] = cmd;
+			report("Failed to execute command: %s", get_status_message(code));
+		} else {
+			request = exec_run_request(view, &req);
+			argv[1] = cmd;
+			return request;
+		}
+
 	} else if (!strcmp(cmd, "toggle")) {
 		enum view_flag flags = VIEW_NO_FLAGS;
 		enum status_code code = prompt_toggle(view, argv, &flags);
 		const char *action = get_status_message(code);
-		int i;
 
 		if (code != SUCCESS) {
 			report("%s", action);
 			return REQ_NONE;
 		}
 
-		if (flags & VIEW_RESET_DISPLAY) {
-			resize_display();
-			redraw_display(TRUE);
-		}
-
-		foreach_displayed_view(view, i) {
-			if (view_has_flags(view, flags) && view_can_refresh(view))
-				reload_view(view);
-			else
-				redraw_view(view);
-		}
+		prompt_update_display(flags);
 
 		if (*action)
 			report("%s", action);
 
+	} else if (!strcmp(cmd, "script")) {
+		enum status_code code = open_script(argv[1]);
+
+		if (code != SUCCESS)
+			report("%s", get_status_message(code));
+		return REQ_NONE;
+
 	} else {
-		struct key key = {};
+		struct key key = {{0}};
+		enum status_code code;
+		enum view_flag flags = VIEW_NO_FLAGS;
 
 		/* Try :<key> */
 		key.modifiers.multibytes = 1;
 		string_ncopy(key.data.bytes, cmd, cmdlen);
-		request = get_keybinding(view->keymap, &key, 1);
-		if (request != REQ_NONE)
+		request = get_keybinding(view->keymap, &key, 1, NULL);
+		if (request != REQ_UNKNOWN)
 			return request;
 
 		/* Try :<command> */
@@ -864,18 +871,95 @@ run_prompt_command(struct view *view, const char *argv[])
 		if (request != REQ_UNKNOWN)
 			return request;
 
-		if (set_option(argv[0], argv_size(argv + 1), &argv[1]) == SUCCESS) {
+		code = set_option(argv[0], argv_size(argv + 1), &argv[1]);
+		if (code != SUCCESS) {
+			report("%s", get_status_message(code));
+			return REQ_NONE;
+		}
+
+		if (!strcmp(cmd, "set")) {
+			struct option_info *toggle;
+
+			toggle = find_option_info(option_toggles, ARRAY_SIZE(option_toggles),
+						  "", argv[1]);
+
+			if (toggle)
+				flags = toggle->flags;
+		}
+
+		if (flags) {
+			prompt_update_display(flags);
+
+		} else {
 			request = view_can_refresh(view) ? REQ_REFRESH : REQ_SCREEN_REDRAW;
 			if (!strcmp(cmd, "color"))
 				init_colors();
 			resize_display();
-			redraw_display(TRUE);
+			redraw_display(true);
 		}
-		return request;
+
 	}
 	return REQ_NONE;
 }
 
+enum request
+exec_run_request(struct view *view, struct run_request *req)
+{
+	const char **argv = NULL;
+	bool confirmed = false;
+	enum request request = REQ_NONE;
+	char cmd[SIZEOF_STR];
+	const char *req_argv[SIZEOF_ARG];
+	int req_argc = 0;
+
+	if (!argv_to_string(req->argv, cmd, sizeof(cmd), " ")
+	    || !argv_from_string_no_quotes(req_argv, &req_argc, cmd)
+	    || !argv_format(view->env, &argv, req_argv, false, true)) {
+		report("Failed to format arguments");
+		return REQ_NONE;
+	}
+
+	if (req->flags.internal) {
+		request = run_prompt_command(view, argv);
+
+	} else {
+		confirmed = !req->flags.confirm;
+
+		if (req->flags.confirm) {
+			char cmd[SIZEOF_STR], prompt[SIZEOF_STR];
+			const char *and_exit = req->flags.exit ? " and exit" : "";
+
+			if (argv_to_string_quoted(argv, cmd, sizeof(cmd), " ") &&
+			    string_format(prompt, "Run `%s`%s?", cmd, and_exit) &&
+			    prompt_yesno(prompt)) {
+				confirmed = true;
+			}
+		}
+
+		if (confirmed)
+			open_external_viewer(argv, repo.cdup, req->flags.silent,
+					     !req->flags.exit, false, "");
+	}
+
+	if (argv)
+		argv_free(argv);
+	free(argv);
+
+	if (request == REQ_NONE) {
+		if (req->flags.confirm && !confirmed)
+			request = REQ_NONE;
+
+		else if (req->flags.exit)
+			request = REQ_QUIT;
+
+		else if (!req->flags.internal && watch_dirty(&view->watch))
+			request = REQ_REFRESH;
+
+	}
+
+	return request;
+}
+
 enum request
 open_prompt(struct view *view)
 {
