@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -46,7 +46,7 @@ refs_get_column_data(struct view *view, const struct line *line, struct view_col
 	column_data->ref = reference->ref;
 	column_data->commit_title = reference->title;
 
-	return TRUE;
+	return true;
 }
 
 static enum request
@@ -56,7 +56,7 @@ refs_request(struct view *view, enum request request, struct line *line)
 
 	switch (request) {
 	case REQ_REFRESH:
-		load_refs(TRUE);
+		load_refs(true);
 		refresh_view(view);
 		return REQ_NONE;
 
@@ -64,11 +64,16 @@ refs_request(struct view *view, enum request request, struct line *line)
 	{
 		const struct ref *ref = reference->ref;
 		const char *all_references_argv[] = {
-			GIT_MAIN_LOG_CUSTOM(encoding_arg, commit_order_arg(), "",
-				refs_is_all(reference) ? "--all" : ref->name, "")
+			GIT_MAIN_LOG(encoding_arg, commit_order_arg(),
+				"%(mainargs)", "",
+				refs_is_all(reference) ? "--all" : ref->name, "",
+				log_custom_pretty_arg())
 		};
 
-		open_argv(view, &main_view, all_references_argv, NULL, OPEN_SPLIT);
+		if (!argv_format(main_view.env, &main_view.argv, all_references_argv, false, false))
+			report("Failed to format argument");
+		else
+			open_view(view, &main_view, OPEN_SPLIT | OPEN_PREPARED);
 		return REQ_NONE;
 	}
 	default:
@@ -77,18 +82,18 @@ refs_request(struct view *view, enum request request, struct line *line)
 }
 
 static bool
-refs_read(struct view *view, struct buffer *buf)
+refs_read(struct view *view, struct buffer *buf, bool force_stop)
 {
-	struct reference template = {};
+	struct reference template = {0};
 	char *author;
 	char *title;
 	size_t i;
 
 	if (!buf)
-		return TRUE;
+		return true;
 
 	if (!*buf->data)
-		return FALSE;
+		return false;
 
 	author = io_memchr(buf, buf->data, 0);
 	title = io_memchr(buf, author, 0);
@@ -106,13 +111,13 @@ refs_read(struct view *view, struct buffer *buf)
 		reference->time = template.time;
 
 		if (title)
-			string_expand(reference->title, sizeof(reference->title), title, 1);
+			string_expand(reference->title, sizeof(reference->title), title, strlen(title), 1);
 
-		view->line[i].dirty = TRUE;
+		view->line[i].dirty = true;
 		view_column_info_update(view, &view->line[i]);
 	}
 
-	return TRUE;
+	return true;
 }
 
 static bool
@@ -125,12 +130,12 @@ refs_open_visitor(void *data, const struct ref *ref)
 
 	line = add_line_alloc(view, &reference, LINE_DEFAULT, 0, is_all);
 	if (!line)
-		return FALSE;
+		return false;
 
 	reference->ref = ref;
 	view_column_info_update(view, line);
 
-	return TRUE;
+	return true;
 }
 
 static bool
@@ -138,7 +143,8 @@ refs_open(struct view *view, enum open_flags flags)
 {
 	const char *refs_log[] = {
 		"git", "log", encoding_arg, "--no-color", "--date=raw",
-			"--pretty=format:%H%x00%an <%ae> %ad%x00%s",
+			opt_mailmap ? "--pretty=format:%H%x00%aN <%aE> %ad%x00%s"
+				    : "--pretty=format:%H%x00%an <%ae> %ad%x00%s",
 			"--all", "--simplify-by-decoration", NULL
 	};
 
@@ -153,15 +159,18 @@ refs_open(struct view *view, enum open_flags flags)
 
 	if (!refs_all || !begin_update(view, NULL, refs_log, OPEN_RELOAD)) {
 		report("Failed to load reference data");
-		return FALSE;
+		return false;
 	}
 
+	if (!view->lines)
+		view->sort.current = get_view_column(view, VIEW_COLUMN_REF);
 	refs_open_visitor(view, refs_all);
 	foreach_ref(refs_open_visitor, view);
+	resort_view(view, true);
 
 	watch_register(&view->watch, WATCH_HEAD | WATCH_REFS);
 
-	return TRUE;
+	return true;
 }
 
 static void
@@ -175,8 +184,8 @@ refs_select(struct view *view, struct line *line)
 	}
 	string_copy_rev(view->ref, reference->ref->id);
 	string_copy_rev(view->env->head, reference->ref->id);
-	string_copy_rev(view->env->ref, reference->ref->name);
-	ref_update_env(view->env, reference->ref, TRUE);
+	string_ncopy(view->env->ref, reference->ref->name, strlen(reference->ref->name));
+	ref_update_env(view->env, reference->ref, false);
 }
 
 static struct view_ops refs_ops = {
